{"cells": [{"cell_type": "markdown", "id": "bb51ec39", "metadata": {}, "source": ["# Methylcyclohexane A-value\n", "\n", "In this exercise we will compute the A-Value of the methyl group in a methylcyclohexane molecule. A-values are numerical values used in the determination of the most stable orientation of atoms in a molecule, derived from energy measurements of the different cyclohexane conformations of a monosubstituted cyclohexane chemical. In particular, we will compare a cyclohexane monosubstituted with a methyl group  in equatorial or axial position.\n", "\n", "```{figure} ../../images/aval.png\n", "---\n", "name: MeC6H11conformers\n", "---\n", "Conformers considered to compute A-value for a methyl group. [Image source](https://en.wikipedia.org/wiki/File:MeC6H11conformers.svg)\n", "```\n"]}, {"cell_type": "markdown", "id": "224d5b1a", "metadata": {}, "source": ["## Creating the conformers\n", "\n", "For larger molecules, the creation of an approximate xyz file that can\n", "be used as the molecular input coordinates is impossible. In these\n", "cases, one may refer to molecular visualization softwares. \n", "\n", "Many different programs can be used for this task. Popular ones are for example Avogadro (open source, cross-plattform), \n", "*MolView* (https://molview.org/) (web-based), molden, GaussView (comes with the Gaussian quantum chemistry software), IQMol and many more.  \n", "\n", "An alternative is to download structures from a database such as PubChem. We will do this for methylcyclohexane in equatorial position.\n", "\n", "Concerning the methylcyclohexane in axial position, for simplicity we provide you the coordinates below, but you could as well have drawn them yourselves. The coordinates need to be approximately right and many programs come with a force field optimization routine, that will quickly snap together all bonds to their equilibrium distances and so on. \n"]}, {"cell_type": "code", "execution_count": 1, "id": "ec219553", "metadata": {}, "outputs": [], "source": ["import psi4\n", "import pandas as pd\n", "\n", "import sys\n", "sys.path.append(\"..\")\n", "\n", "from helpers import *"]}, {"cell_type": "code", "execution_count": 2, "id": "8dc07578", "metadata": {}, "outputs": [{"name": "stdout", "output_type": "stream", "text": ["  Threads set to 2 by Python driver.\n", "\n", "  Memory set to   1.863 GiB by Python driver.\n"]}, {"data": {"text/plain": ["2000000000"]}, "execution_count": 2, "metadata": {}, "output_type": "execute_result"}], "source": ["psi4.set_num_threads(2)\n", "psi4.set_memory('2 GB')"]}, {"cell_type": "code", "execution_count": 3, "id": "50ec174b", "metadata": {}, "outputs": [{"name": "stdout", "output_type": "stream", "text": ["\tSearching PubChem database for methylcyclohexane (single best match returned)\n", "\tFound 1 result(s)\n"]}], "source": ["equatorial = psi4.geometry(\"pubchem:methylcyclohexane\")"]}, {"cell_type": "code", "execution_count": 4, "id": "524412ff", "metadata": {}, "outputs": [], "source": ["axial = psi4.geometry(\"\"\"\n", "0 1\n", "            C    -1.2274001186   0.0240644681   0.0713205260\n", "            C    -0.4303137771   1.2916249494   0.4268585314\n", "            C    -0.4722293117  -1.2264891935   0.5553779250\n", "            C     1.0230602330   1.2466021870  -0.0584805092\n", "            C     0.9810630014  -1.2793844777   0.0705620697\n", "            C     1.7419564780  -0.0059874325   0.4503640869\n", "            C    -1.5835089284  -0.0460222638  -1.4189886323\n", "            H    -2.1693676159   0.0675361033   0.6153188260\n", "            H    -0.4294212171   1.4082548027   1.5099113283\n", "            H    -0.9316553296   2.1691116345   0.0246221983\n", "            H    -1.0030678716  -2.1232152029   0.2435950228\n", "            H    -0.4734758600  -1.2323353337   1.6446806077\n", "            H     1.5465084148   2.1388141117   0.2762673484\n", "            H     1.0547234511   1.2652076515  -1.1458128465\n", "            H     1.0103574383  -1.4097971319  -1.0091493740\n", "            H     1.4751656749  -2.1496795922   0.4954590794\n", "            H     2.7555071747  -0.0428481064   0.0586760240\n", "            H     1.8302774578   0.0479649801   1.5350921581\n", "            H    -2.1583756764   0.8264696291  -1.7176644216\n", "            H    -0.7081555279  -0.0931653015  -2.0577965825\n", "            H    -2.1875114783  -0.9248063650  -1.6283284158\n", "\"\"\")\n"]}, {"cell_type": "code", "execution_count": 5, "id": "01b3a79c", "metadata": {}, "outputs": [{"data": {"application/3dmoljs_load.v0": "<div id=\"3dmolviewer_16696464473298204\"  style=\"position: relative; width: 640px; height: 480px\">\n        <p id=\"3dmolwarning_16696464473298204\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n        </div>\n<script>\n\nvar loadScriptAsync = function(uri){\n  return new Promise((resolve, reject) => {\n    var tag = document.createElement('script');\n    tag.src = uri;\n    tag.async = true;\n    tag.onload = () => {\n      resolve();\n    };\n  var firstScriptTag = document.getElementsByTagName('script')[0];\n  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n});\n};\n\nif(typeof $3Dmolpromise === 'undefined') {\n$3Dmolpromise = null;\n  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n}\n\nvar viewer_16696464473298204 = null;\nvar warn = document.getElementById(\"3dmolwarning_16696464473298204\");\nif(warn) {\n    warn.parentNode.removeChild(warn);\n}\n$3Dmolpromise.then(function() {\nvar viewergrid_16696464473298204 = null;\nviewergrid_16696464473298204 = $3Dmol.createViewerGrid($(\"#3dmolviewer_16696464473298204\"),{rows: 1, cols: 2, control_all: true},{backgroundColor:\"white\"});\nviewer_16696464473298204 = viewergrid_16696464473298204[0][0];\nviewergrid_16696464473298204[0][0].zoomTo();viewergrid_16696464473298204[0][1].zoomTo();\tviewergrid_16696464473298204[0][0].addModel(\"21\\n\\n C   -1.020484191824    0.000002054489   -0.330316577404\\n C   -0.306284191824    1.263702054489    0.168083422596\\n C   -0.306384191824   -1.263597945511    0.167983422596\\n C    1.173015808176    1.261502054489   -0.210616577404\\n C    1.172915808176   -1.261597945511   -0.210516577404\\n C    1.877315808176    0.000002054489    0.278783422596\\n C   -2.484384191824    0.000002054489    0.106283422596\\n H   -0.996284191824    0.000002054489   -1.428316577404\\n H   -0.398884191824    1.339202054489    1.259183422596\\n H   -0.785784191824    2.155202054489   -0.253516577404\\n H   -0.785884191824   -2.155097945511   -0.253616577404\\n H   -0.399084191824   -1.339197945511    1.259083422596\\n H    1.663115808176    2.145102054489    0.213483422596\\n H    1.270415808176    1.334502054489   -1.300816577404\\n H    1.270415808176   -1.334797945511   -1.300716577404\\n H    1.662915808176   -2.145197945511    0.213783422596\\n H    2.916915808176   -0.000097945511   -0.067516577404\\n H    1.905515808176    0.000002054489    1.375383422596\\n H   -3.003484191824    0.885002054489   -0.276616577404\\n H   -3.003684191824   -0.884797945511   -0.276616577404\\n H   -2.574884191824    0.000002054489    1.197783422596\\n\",\"xyz\");\n\tviewergrid_16696464473298204[0][1].addModel(\"21\\n\\n C   -1.227400118597    0.024064468079    0.071320525994\\n C   -0.430313777097    1.291624949379    0.426858531394\\n C   -0.472229311697   -1.226489193521    0.555377924994\\n C    1.023060233003    1.246602186979   -0.058480509206\\n C    0.981063001403   -1.279384477721    0.070562069694\\n C    1.741956478003   -0.005987432521    0.450364086894\\n C   -1.583508928397   -0.046022263821   -1.418988632306\\n H   -2.169367615897    0.067536103279    0.615318825994\\n H   -0.429421217097    1.408254802679    1.509911328294\\n H   -0.931655329597    2.169111634479    0.024622198294\\n H   -1.003067871597   -2.123215202921    0.243595022794\\n H   -0.473475859997   -1.232335333721    1.644680607694\\n H    1.546508414803    2.138814111679    0.276267348394\\n H    1.054723451103    1.265207651479   -1.145812846506\\n H    1.010357438303   -1.409797131921   -1.009149374006\\n H    1.475165674903   -2.149679592221    0.495459079394\\n H    2.755507174703   -0.042848106421    0.058676023994\\n H    1.830277457803    0.047964980079    1.535092158094\\n H   -2.158375676397    0.826469629079   -1.717664421606\\n H   -0.708155527897   -0.093165301521   -2.057796582506\\n H   -2.187511478297   -0.924806365021   -1.628328415806\\n\",\"xyz\");\n\tviewergrid_16696464473298204[0][0].setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n\tviewergrid_16696464473298204[0][1].setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n\tviewergrid_16696464473298204[0][0].zoomTo();\n\tviewergrid_16696464473298204[0][1].zoomTo();\nviewergrid_16696464473298204[0][1].render();\nviewergrid_16696464473298204[0][0].render();\n});\n</script>", "text/html": ["<div id=\"3dmolviewer_16696464473298204\"  style=\"position: relative; width: 640px; height: 480px\">\n", "        <p id=\"3dmolwarning_16696464473298204\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n", "        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n", "        </div>\n", "<script>\n", "\n", "var loadScriptAsync = function(uri){\n", "  return new Promise((resolve, reject) => {\n", "    var tag = document.createElement('script');\n", "    tag.src = uri;\n", "    tag.async = true;\n", "    tag.onload = () => {\n", "      resolve();\n", "    };\n", "  var firstScriptTag = document.getElementsByTagName('script')[0];\n", "  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n", "});\n", "};\n", "\n", "if(typeof $3Dmolpromise === 'undefined') {\n", "$3Dmolpromise = null;\n", "  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n", "}\n", "\n", "var viewer_16696464473298204 = null;\n", "var warn = document.getElementById(\"3dmolwarning_16696464473298204\");\n", "if(warn) {\n", "    warn.parentNode.removeChild(warn);\n", "}\n", "$3Dmolpromise.then(function() {\n", "var viewergrid_16696464473298204 = null;\n", "viewergrid_16696464473298204 = $3Dmol.createViewerGrid($(\"#3dmolviewer_16696464473298204\"),{rows: 1, cols: 2, control_all: true},{backgroundColor:\"white\"});\n", "viewer_16696464473298204 = viewergrid_16696464473298204[0][0];\n", "viewergrid_16696464473298204[0][0].zoomTo();viewergrid_16696464473298204[0][1].zoomTo();\tviewergrid_16696464473298204[0][0].addModel(\"21\\n\\n C   -1.020484191824    0.000002054489   -0.330316577404\\n C   -0.306284191824    1.263702054489    0.168083422596\\n C   -0.306384191824   -1.263597945511    0.167983422596\\n C    1.173015808176    1.261502054489   -0.210616577404\\n C    1.172915808176   -1.261597945511   -0.210516577404\\n C    1.877315808176    0.000002054489    0.278783422596\\n C   -2.484384191824    0.000002054489    0.106283422596\\n H   -0.996284191824    0.000002054489   -1.428316577404\\n H   -0.398884191824    1.339202054489    1.259183422596\\n H   -0.785784191824    2.155202054489   -0.253516577404\\n H   -0.785884191824   -2.155097945511   -0.253616577404\\n H   -0.399084191824   -1.339197945511    1.259083422596\\n H    1.663115808176    2.145102054489    0.213483422596\\n H    1.270415808176    1.334502054489   -1.300816577404\\n H    1.270415808176   -1.334797945511   -1.300716577404\\n H    1.662915808176   -2.145197945511    0.213783422596\\n H    2.916915808176   -0.000097945511   -0.067516577404\\n H    1.905515808176    0.000002054489    1.375383422596\\n H   -3.003484191824    0.885002054489   -0.276616577404\\n H   -3.003684191824   -0.884797945511   -0.276616577404\\n H   -2.574884191824    0.000002054489    1.197783422596\\n\",\"xyz\");\n", "\tviewergrid_16696464473298204[0][1].addModel(\"21\\n\\n C   -1.227400118597    0.024064468079    0.071320525994\\n C   -0.430313777097    1.291624949379    0.426858531394\\n C   -0.472229311697   -1.226489193521    0.555377924994\\n C    1.023060233003    1.246602186979   -0.058480509206\\n C    0.981063001403   -1.279384477721    0.070562069694\\n C    1.741956478003   -0.005987432521    0.450364086894\\n C   -1.583508928397   -0.046022263821   -1.418988632306\\n H   -2.169367615897    0.067536103279    0.615318825994\\n H   -0.429421217097    1.408254802679    1.509911328294\\n H   -0.931655329597    2.169111634479    0.024622198294\\n H   -1.003067871597   -2.123215202921    0.243595022794\\n H   -0.473475859997   -1.232335333721    1.644680607694\\n H    1.546508414803    2.138814111679    0.276267348394\\n H    1.054723451103    1.265207651479   -1.145812846506\\n H    1.010357438303   -1.409797131921   -1.009149374006\\n H    1.475165674903   -2.149679592221    0.495459079394\\n H    2.755507174703   -0.042848106421    0.058676023994\\n H    1.830277457803    0.047964980079    1.535092158094\\n H   -2.158375676397    0.826469629079   -1.717664421606\\n H   -0.708155527897   -0.093165301521   -2.057796582506\\n H   -2.187511478297   -0.924806365021   -1.628328415806\\n\",\"xyz\");\n", "\tviewergrid_16696464473298204[0][0].setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n", "\tviewergrid_16696464473298204[0][1].setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n", "\tviewergrid_16696464473298204[0][0].zoomTo();\n", "\tviewergrid_16696464473298204[0][1].zoomTo();\n", "viewergrid_16696464473298204[0][1].render();\n", "viewergrid_16696464473298204[0][0].render();\n", "});\n", "</script>"]}, "metadata": {}, "output_type": "display_data"}], "source": ["drawXYZSideBySide(equatorial, axial)"]}, {"cell_type": "markdown", "id": "4c459ab5", "metadata": {}, "source": ["\n", ":::{admonition} Exercise 1\n", ":class: exercise \n", "Why is it important to construct the starting structures in a specific conformation, although one is carrying out a geometry optimisation? Don't we risk to end up in the same conformation after the optimization, since we are dealing in both cases with the same molecule, i.e. methylcyclohexane?\n", ":::\n", "\n"]}, {"cell_type": "code", "execution_count": 6, "id": "0b045698", "metadata": {}, "outputs": [], "source": ["results_dict = {}"]}, {"cell_type": "markdown", "id": "3292d8d9", "metadata": {}, "source": [":::{admonition} Hint\n", ":class: tip\n", "You can generate a table using python by creating a list of the methods you want to use and iterating over them. \n", "As we want to start the geometry optimizations from the same geometries it is important that you always optimize a clone of the original molecule. \n", "We can do this by using `mol_to_optimize =  starting_mol.clone()`. \n", "To display the table you need to store the results in a dictionary. \n", "\n", "A dictionary in python looks like this: `results_dict = {\"key\": \"value\"}`. The value can be a `str`, `int` or even a `list`. \n", "In this case you can update the dictionary for each method iteration like so:\n", "`results_dict[method] = [E1, E2, dH, label]`\n", "\n", "In the end you will have a full dictionary containing all the results which we can nicely print as a pandas dataframe using:\n", "\n", "`pd.DataFrame.from_dict(data, orient='index', columns=['E_reactant', 'E_product', '\u0394H(0K) [kcal/mol]', 'remark'])`\n", ":::"]}, {"cell_type": "code", "execution_count": 8, "id": "92144c06", "metadata": {}, "outputs": [], "source": ["psi4.core.set_output_file(f'a-val.log', False)"]}, {"cell_type": "markdown", "id": "91a35429", "metadata": {}, "source": [":::{admonition} Exercise 2\n", ":class: exercise \n", "Compute the energy of the two conformations and calculate the energy difference at 0 K, $\\Delta E_{0K} = E_{\\text{axial}} - E_{\\text{equatorial}}$, i.e. the A-value, filling the table below. Compare the results of different methods with the experimental value for the  A-Value of the methil group in a methylcyclohexane molecule, **1.74 kcal/mol**. For each method, use both 6-31+G* and 6-31+G** basis sets.\n", "\n", "By adding `%%time` at the beginning of each cell, the execution time will be printed out. This quantity is related to the computational cost of the calculation and can help you to compare the different methods.\n", "\n", "\n", "**You can split up the calculations among yourselves to speed up the process.**\n", "\n", "\n", "| Method      | E_equatorial (Hartrees) | E_axial (Hartrees) | \u0394E(0K) (kcal/mol) | Relative error % | Basis set | Remark (\u03c8 /\u03c1 based ?) | Wall time | \n", "| ----------- | ----------------------- | ------------------ | ----------------- | ---------------- | --------- | --------------------- | --------- |\n", "| HF          |  |  |  |  |  |  |  |  |\n", "| MP2         |  |  |  |  |  |  |  |  |\n", "| BLYP        |  |  |  |  |  |  |  |  |\n", "| MPW1PW91    |  |  |  |  |  |  |  |  |\n", "| B97-2       |  |  |  |  |  |  |  |  |\n", "| PBE         |  |  |  |  |  |  |  |  |\n", "| TPSS        |  |  |  |  |  |  |  |  |\n", "| M06-L       |  |  |  |  |  |  |  |  |\n", "| M06-2X      |  |  |  |  |  |  |  |  |\n", "| B3LYP-D3BJ  |  |  |  |  |  |  |  |  |\n", "| Reference   | //                      | //                 | 1.74              | Experimental         |\n", "\n", ":::\n"]}, {"cell_type": "code", "execution_count": null, "id": "7c3f6178", "metadata": {}, "outputs": [], "source": ["%%time\n", "methods =  [ ]  # add methods here\n", "                # required methods:  'hf', 'mp2', 'blyp', 'mpw1pw91','b97-2','PBE', 'TPSS', 'M06-L', 'M06-2X', 'B3LYP-D3BJ'\n", "                # required basis sets: '6-31+G*', '6-31+G**'\n", "# clone molecules\n", "equatorial_hf = equatorial.clone()\n", "axial_hf = axial.clone()\n", "\n", "for method in methods:\n", "    product_dft = equatorial.clone()\n", "    axial_dft = axial.clone()\n", "    \n", "    psi4.core.clean_options() # reset options for each run\n", "\n", "    if method == \"mp2\": # for the mp2 method we will use the non-density fitted MP2 version which not the default in MP2\n", "        psi4.set_options({'mp2_type':'conv'})\n", "        \n", "    # perform geometry optimizations and save into variables\n", "    E_equatorial = None #\n", "    E_axial = None #\n", "    \n", "    results_dict[method] = None # You can e.g use a list like this [E_equatorial, E_axial, \u0394E_kcalmol,  \"\u03c1/\u03c8 based\"]"]}, {"cell_type": "code", "execution_count": 10, "id": "d29c1f9d", "metadata": {}, "outputs": [], "source": ["# we insert the reference here\n", "results_dict['reference'] = ['-','-', 1.74, 'exp']"]}, {"cell_type": "code", "execution_count": null, "id": "ff65dd5c", "metadata": {}, "outputs": [], "source": ["# print out dictionary nicely \n", "# FYI: you can also print it to latex code by adding .to_latex() and using \\usepackage{booktabs}\n", "pd.DataFrame.from_dict(results_dict, orient='index', columns=['E_equatorial', 'E_axial', '\u0394E(0K) [kcal/mol]', 'remark'])"]}, {"cell_type": "markdown", "id": "44d77ab3", "metadata": {}, "source": ["\n", ":::{admonition} Exercise 3\n", ":class: exercise \n", "What kind of basis sets were used for \u03c8 based calculations? On which atoms will polarization and diffuse functions be included? Are we close to the basis set limit? Why, or why not?\n", ":::"]}, {"cell_type": "markdown", "id": "f30eb231", "metadata": {}, "source": [":::{admonition} Exercise 4\n", ":class: exercise \n", "Comment on the performance of the wavefunction based methods and DFT, comparing both accuracy and computational time. \n", "\n", "Can you imagine why some DFT methods perform better than others? How does DFT compare to MP2 and HF? \n", "Would you expect the exchange-correlation functionals to give similar errors in different systems?\n", "\n", "An often cited target for the development of exchange-correlation functionals is an error below *chemical accuracy*, *i.e.* within 1\n", "    kcal mol $^{-1}$ of the accurate result. Explain the accuracy of the results also in light of this.\n", ":::\n"]}], "metadata": {"celltoolbar": "Tags", "hide_input": false, "kernelspec": {"display_name": "Computational Chemistry", "language": "python", "name": "compchem"}, "language_info": {"codemirror_mode": {"name": "ipython", "version": 3}, "file_extension": ".py", "mimetype": "text/x-python", "name": "python", "nbconvert_exporter": "python", "pygments_lexer": "ipython3", "version": "3.8.12"}, "unianalytics_cell_mapping": [["bb51ec39", "bb51ec39"], ["224d5b1a", "224d5b1a"], ["ec219553", "ec219553"], ["8dc07578", "8dc07578"], ["50ec174b", "50ec174b"], ["524412ff", "524412ff"], ["01b3a79c", "01b3a79c"], ["4c459ab5", "4c459ab5"], ["8b816917", "8b816917"], ["0b045698", "0b045698"], ["3292d8d9", "3292d8d9"], ["92144c06", "92144c06"], ["91a35429", "91a35429"], ["d75ada96", "d75ada96"], ["7c3f6178", "7c3f6178"], ["d29c1f9d", "d29c1f9d"], ["ff65dd5c", "ff65dd5c"], ["3fc29fb1", "3fc29fb1"], ["c2878137", "c2878137"], ["6f6ae489", "6f6ae489"], ["a15de3e3", "a15de3e3"], ["b070945b", "b070945b"], ["2afbeebf", "2afbeebf"], ["44d77ab3", "44d77ab3"], ["ea4184bd", "ea4184bd"], ["f30eb231", "f30eb231"], ["8287bbb2", "8287bbb2"]], "unianalytics_notebook_id": "a80a4182-c0d2-42f8-855f-2a865caa9620", "varInspector": {"cols": {"lenName": 16, "lenType": 16, "lenVar": 40}, "kernels_config": {"python": {"delete_cmd_postfix": "", "delete_cmd_prefix": "del ", "library": "var_list.py", "varRefreshCmd": "print(var_dic_list())"}, "r": {"delete_cmd_postfix": ") ", "delete_cmd_prefix": "rm(", "library": "var_list.r", "varRefreshCmd": "cat(var_dic_list()) "}}, "types_to_exclude": ["module", "function", "builtin_function_or_method", "instance", "_Feature"], "window_display": false}}, "nbformat": 4, "nbformat_minor": 5}